// SPDX-FileCopyrightText: 2021 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

.markdown {
  > p,
  > hr,
  > ul,
  > ol,
  > table,
  > pre,
  > blockquote,
  li > ul,
  li > ol {
    margin: 1em 0 1em 0;

    &:first-child {
      margin-top: 0;
    }
  }

  p {
    &:last-child {
      margin-bottom: 0;
    }
  }

  img {
    max-width: 100%;
    height: auto;
  }

  a {
    border-bottom: 1px dotted @text-color-light;

    &:hover, &:focus {
      border-bottom: 1px solid @text-color;
      text-decoration: none;
    }

    img {
      max-width: 32em;
    }

    &.with-thumbnail {
      img {
        padding: 1px;
      }

      &:hover, &:focus {
        img {
          padding: 0;
        }
      }
    }
  }

  table {
    border-collapse: collapse;

    th {
      text-align: left;
      background-color: @gray-lighter;
    }

    &, th, td {
      border: 1px solid;
      border-color: @gray-light;
    }
  }
}

.markdown.inline {
  img {
    max-height: 100%;
    vertical-align: middle;
  }

  a.with-thumbnail {
    &, &:hover, &:focus {
      border-bottom: none;
    }
  }
}
