// SPDX-FileCopyrightText: 2021 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

.action-list {
  [data-action-item]:hover {
    background-color: @tr-hover-color;
    cursor: pointer;
  }

  [data-action-item].active {
    background-color: @tr-active-color;
  }

  &[data-icinga-multiselect-url] * {
    user-select: none;
  }
}

#layout.minimal-layout .footer > .selection-count {
  display: none;
}
