// SPDX-FileCopyrightText: 2020 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

.key-value-list {
  list-style-type: none;
  margin: 0;
  padding: 0;

  li {
    display: flex;
  }

  li > span {
    padding: .25em;

    &.label {
      display: block;
      padding-left: 0;
      width: 12em;
    }
  }
}
