// SPDX-FileCopyrightText: 2021 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

.performance-data-table {
  width: 100%;
  overflow-x: auto;
  display: block;

  tr:not(:last-child) {
    border-bottom: 1px solid @gray-lighter;
  }

  td {
    text-align: right;
    .text-ellipsis();
  }

  th {
    font-size: .857em;
    font-weight: normal;
    text-transform: uppercase;
    letter-spacing: .05em;
  }

  thead {
    border-bottom: 1px solid @gray-light;
  }

  th:first-child,
  td:first-child {
    padding-left: 0;
  }

  .title {
    text-align: left;
    width: 100%;
  }

  td.title {
    font-weight: bold;
  }

  .sparkline-col {
    min-width: 1.75em;
    width: 1.75em;
    padding: 2/12em 0;
    .invalid-perfdata {
      font-size: 1.25em;
      vertical-align: text-bottom;
      color: @color-warning;
    }
  }

  .inline-pie > svg {
    vertical-align: middle;
  }

  .no-value {
    color: @gray-semilight;
    text-align: center;
    display: block;
  }
}
