<?php

// SPDX-FileCopyrightText: 2019 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

namespace Icinga\Module\Icingadb\Widget;

/** @deprecated Use {@see \ipl\Web\Widget\StateBadge} instead */
class StateBadge extends \ipl\Web\Widget\StateBadge
{
}
