<?php

// SPDX-FileCopyrightText: 2021 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

namespace Icinga\Module\Icingadb\Command\Transport;

use Icinga\Exception\IcingaException;

/**
 * Exception thrown if a command was not successful
 */
class ApiCommandException extends IcingaException
{
}
