// SPDX-FileCopyrightText: 2019 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

.monospace() {
  font-family: SFMono-Regular, Consolas, "Liberation Mono", Menlo, monospace;
}
