// SPDX-FileCopyrightText: 2020 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

.tag-list {
  line-height: 1.5;
  list-style-type: none;
  margin: -.25em 0 0 0; // TODO: This is wrong here, if at all this must be part of wherever this widget is placed
  padding: 0;

  > li {
    display: inline-block;

    &:not(:last-child) {
      margin-right: .417em;
      margin-bottom: .25em; // TODO: Really? It's an inline ul, bottom margin is outer layout..
    }

    i {
      opacity: 0.8;
    }
  }

  > li > a {
    .rounded-corners();
    background-color: @gray-lighter;
    display: block;
    padding: .25em .5em;

    &:hover {
      background-color: @gray-light;
      text-decoration: none;
    }
  }
}
