// SPDX-FileCopyrightText: 2024 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

.dependency-node-state-badges {
  .state-badges();

  .state-badge  {
    &.state-problem {
      background-color: @color-critical;
    }
  }
}
